use crate::common::{load_input, parse_nums};

fn parse_input(lines: &[String]) -> (Vec<u64>, Vec<u64>) {
    (
        parse_nums(lines.get(0).unwrap()),
        parse_nums(lines.get(1).unwrap()),
    )
}

fn distance_by_time_pressed(time_pressed: u64, race_time: u64) -> u64 {
    (race_time - time_pressed) * time_pressed
}

fn join_nums(nums: &[u64]) -> u64 {
    nums.iter()
        .map(|e| e.to_string())
        .collect::<String>()
        .parse()
        .unwrap()
}

pub fn run_part2() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/6.txt")?;
    let (time, distance) = parse_input(&lines);
    let new_time = join_nums(&time);
    let new_dist = join_nums(&distance);

    let res = (0..new_time)
        .map(|pressed| distance_by_time_pressed(pressed, new_time))
        .filter(|e| *e > new_dist)
        .count();

    Ok(res.try_into().unwrap())
}

pub fn run() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/6.txt")?;
    let (time, distance) = parse_input(&lines);

    let res = time
        .iter()
        .zip(distance.iter())
        .map(|e| {
            (0..*e.0)
                .map(|pressed| distance_by_time_pressed(pressed, *e.0))
                .filter(|dist| *dist > *e.1)
                .count()
        })
        .reduce(|acc, e| e * acc)
        .unwrap();

    Ok(res.try_into().unwrap())
}
